@@ -1,34 +1,33 @@
-# -*- coding: utf-8 -*-
 """The base class for xonsh shell"""
 import io
 import os
 import sys
 import time
 
+from xonsh.ansi_colors import ansi_partial_color_format
 from xonsh.built_ins import XSH
+from xonsh.codecache import (
+    code_cache_check,
+    code_cache_name,
+    get_cache_filename,
+    run_compiled_code,
+    should_use_cache,
+    update_cache,
+)
+from xonsh.completer import Completer
+from xonsh.events import events
+from xonsh.lazyimps import pyghooks, pygments
+from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
+from xonsh.prompt.base import PromptFormatter, multiline_prompt
+from xonsh.shell import transform_command
 from xonsh.tools import (
-    XonshError,
-    print_exception,
     DefaultNotGiven,
+    XonshError,
     check_for_partial_string,
     format_std_prepost,
     get_line_continuation,
+    print_exception,
 )
-from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
-from xonsh.codecache import (
-    should_use_cache,
-    code_cache_name,
-    code_cache_check,
-    get_cache_filename,
-    update_cache,
-    run_compiled_code,
-)
-from xonsh.completer import Completer
-from xonsh.prompt.base import multiline_prompt, PromptFormatter
-from xonsh.events import events
-from xonsh.shell import transform_command
-from xonsh.lazyimps import pygments, pyghooks
-from xonsh.ansi_colors import ansi_partial_color_format
 
 if ON_WINDOWS:
     import ctypes
@@ -192,8 +191,10 @@ class _TeeStd(io.TextIOBase):
 
     def flush(self):
         """Flushes both the original stdout and the buffer."""
-        self.std.flush()
-        self.mem.flush()
+        # In certain cases, `std` or `mem` may already be `None` when we're
+        # cleaning up in which case we don't care whether `flush` does anything
+        getattr(getattr(self, "std", lambda: None), "flush", lambda: None)()
+        getattr(getattr(self, "mem", lambda: None), "flush", lambda: None)()
 
     def fileno(self):
         """Tunnel fileno() calls to the std stream."""
@@ -299,11 +300,18 @@ class Tee:
         return s
 
 
-class BaseShell(object):
+class BaseShell:
     """The xonsh shell."""
 
     def __init__(self, execer, ctx, **kwargs):
-        super().__init__()
+        """
+
+        Notes
+        -----
+        classes inheriting multiple base classes should call them explicitly
+        as done for ``ReadlineShell``
+        """
+
         self.execer = execer
         self.ctx = ctx
         self.completer = Completer() if kwargs.get("completer", True) else None
@@ -350,7 +358,10 @@ class BaseShell(object):
 
     def precmd(self, line):
         """Called just before execution of line."""
-        self.precwd = os.getcwd()
+        try:
+            self.precwd = os.getcwd()
+        except FileNotFoundError:
+            self.precwd = os.path.expanduser("~")
         return line if self.need_more_lines else line.lstrip()
 
     def default(self, line, raw_line=None):
@@ -375,7 +386,9 @@ class BaseShell(object):
         tee = Tee(encoding=enc, errors=err)
         ts0 = time.time()
         try:
-            run_compiled_code(code, self.ctx, None, "single")
+            exc_info = run_compiled_code(code, self.ctx, None, "single")
+            if exc_info != (None, None, None):
+                raise exc_info[1]
             ts1 = time.time()
             if hist is not None and hist.last_cmd_rtn is None:
                 hist.last_cmd_rtn = 0  # returncode for success
@@ -383,8 +396,10 @@ class BaseShell(object):
             print(e.args[0], file=sys.stderr)
             if hist is not None and hist.last_cmd_rtn is None:
                 hist.last_cmd_rtn = 1  # return code for failure
-        except Exception:  # pylint: disable=broad-except
-            print_exception()
+        except (SystemExit, KeyboardInterrupt) as err:
+            raise err
+        except BaseException:
+            print_exception(exc_info=exc_info)
             if hist is not None and hist.last_cmd_rtn is None:
                 hist.last_cmd_rtn = 1  # return code for failure
         finally:
@@ -417,6 +432,7 @@ class BaseShell(object):
         """
         hist = XSH.history  # pylint: disable=no-member
         info["rtn"] = hist.last_cmd_rtn if hist is not None else None
+        XSH.env["LAST_RETURN_CODE"] = info["rtn"] or 0
         tee_out = tee_out or None
         last_out = hist.last_cmd_out if hist is not None else None
         if last_out is None and tee_out is None:
@@ -494,7 +510,14 @@ class BaseShell(object):
             self.need_more_lines = True
             return src, None
         try:
-            code = self.execer.compile(src, mode="single", glbs=self.ctx, locs=None)
+            code = self.execer.compile(
+                src,
+                mode="single",
+                glbs=self.ctx,
+                locs=None,
+                filename="<stdin>",
+                compile_empty_tree=False,
+            )
             if _cache:
                 update_cache(code, cachefname)
             self.reset_buffer()
@@ -543,12 +566,16 @@ class BaseShell(object):
             with open(1, "wb", closefd=False) as f:
                 # prevent xonsh from answering interactive questions
                 # on the next command by writing the title
-                f.write("\x1b]0;{0}\x07".format(t).encode())
+                f.write(f"\x1b]0;{t}\x07".encode())
                 f.flush()
 
     @property
     def prompt(self):
         """Obtains the current prompt string."""
+
+        # clear prompt level cache
+        XSH.env["PROMPT_FIELDS"].reset()
+
         if self.need_more_lines:
             if self.mlprompt is None:
                 try:
