@@ -1,37 +1,41 @@
-# -*- coding: utf-8 -*-
 """Import hooks for importing xonsh source files.
 
 This module registers the hooks it defines when it is imported.
 """
-import builtins
 import contextlib
 import importlib
 import os
 import re
 import sys
 import types
-from importlib.abc import MetaPathFinder, SourceLoader, Loader
+from importlib.abc import Loader, MetaPathFinder, SourceLoader
 from importlib.machinery import ModuleSpec
 
+from xonsh.built_ins import XSH
 from xonsh.events import events
 from xonsh.execer import Execer
 from xonsh.lazyasd import lazyobject
 from xonsh.platform import ON_WINDOWS
+from xonsh.tools import print_warning
 
 
 @lazyobject
 def ENCODING_LINE():
     # this regex comes from PEP 263
     # https://www.python.org/dev/peps/pep-0263/#defining-the-encoding
-    return re.compile(b"^[ tv]*#.*?coding[:=][ t]*([-_.a-zA-Z0-9]+)")
+    return re.compile(b"^[ \t\f]*#.*?coding[:=][ \t]*([-_.a-zA-Z0-9]+)")
 
 
 def find_source_encoding(src):
-    """Finds the source encoding given bytes representing a file. If
-    no encoding is found, UTF-8 will be returned as per the docs
+    """Finds the source encoding given bytes representing a file by checking
+    a special comment at either the first or second line of the source file.
     https://docs.python.org/3/howto/unicode.html#unicode-literals-in-python-source-code
+    If no encoding is found, UTF-8 codec with BOM signature will be returned
+    as it skips an optional UTF-8 encoded BOM at the start of the data
+    and is otherwise the same as UTF-8
+    https://docs.python.org/3/library/codecs.html#module-encodings.utf_8_sig
     """
-    utf8 = "UTF-8"
+    utf8 = "utf-8-sig"
     first, _, rest = src.partition(b"\n")
     m = ENCODING_LINE.match(first)
     if m is not None:
@@ -46,26 +50,10 @@ def find_source_encoding(src):
 class XonshImportHook(MetaPathFinder, SourceLoader):
     """Implements the import hook for xonsh source files."""
 
-    def __init__(self, *args, **kwargs):
-        super(XonshImportHook, self).__init__(*args, **kwargs)
+    def __init__(self, execer, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self._filenames = {}
-        self._execer = None
-
-    @property
-    def execer(self):
-        if (
-            hasattr(builtins, "__xonsh__")
-            and hasattr(builtins.__xonsh__, "execer")
-            and builtins.__xonsh__.execer is not None
-        ):
-            execer = builtins.__xonsh__.execer
-            if self._execer is not None:
-                self._execer = None
-        elif self._execer is None:
-            self._execer = execer = Execer(unload=False)
-        else:
-            execer = self._execer
-        return execer
+        self._execer = execer
 
     #
     # MetaPathFinder methods
@@ -87,7 +75,7 @@ class XonshImportHook(MetaPathFinder, SourceLoader):
             if fname not in {x.name for x in os.scandir(p)}:
                 continue
             spec = ModuleSpec(fullname, self)
-            self._filenames[fullname] = os.path.join(p, fname)
+            self._filenames[fullname] = os.path.abspath(os.path.join(p, fname))
             break
         return spec
 
@@ -114,10 +102,10 @@ class XonshImportHook(MetaPathFinder, SourceLoader):
         """Gets the code object for a xonsh file."""
         filename = self.get_filename(fullname)
         if filename is None:
-            msg = "xonsh file {0!r} could not be found".format(fullname)
+            msg = f"xonsh file {fullname!r} could not be found"
             raise ImportError(msg)
         src = self.get_source(fullname)
-        execer = self.execer
+        execer = self._execer
         execer.filename = filename
         ctx = {}  # dummy for modules
         code = execer.compile(src, glbs=ctx, locs=ctx)
@@ -321,7 +309,10 @@ class XonshImportEventLoader(Loader):
         return object.__getattribute__(self, name)
 
 
-def install_import_hooks():
+ARG_NOT_PRESENT = object()
+
+
+def install_import_hooks(execer=ARG_NOT_PRESENT):
     """
     Install Xonsh import hooks in ``sys.meta_path`` in order for ``.xsh`` files
     to be importable and import events to be fired.
@@ -329,6 +320,16 @@ def install_import_hooks():
     Can safely be called many times, will be no-op if xonsh import hooks are
     already present.
     """
+    if execer is ARG_NOT_PRESENT:
+        print_warning(
+            "No execer was passed to install_import_hooks. "
+            "This will become an error in future."
+        )
+        execer = XSH.execer
+        if execer is None:
+            execer = Execer()
+            XSH.load(execer=execer)
+
     found_imp = found_event = False
     for hook in sys.meta_path:
         if isinstance(hook, XonshImportHook):
@@ -336,7 +337,7 @@ def install_import_hooks():
         elif isinstance(hook, XonshImportEventHook):
             found_event = True
     if not found_imp:
-        sys.meta_path.append(XonshImportHook())
+        sys.meta_path.append(XonshImportHook(execer))
     if not found_event:
         sys.meta_path.insert(0, XonshImportEventHook())
 
